-- Copyright (C) 2023:
--   Helmholtz-Zentrum Potsdam Deutsches GeoForschungsZentrum GFZ
--
-- This program is free software: you can redistribute it and/or modify it
-- under the terms of the GNU Affero General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or (at
-- your option) any later version.
--
-- This program is distributed in the hope that it will be useful, but
-- WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Affero
-- General Public License for more details.
--
-- You should have received a copy of the GNU Affero General Public License
-- along with this program. If not, see http://www.gnu.org/licenses/.

-- Converts latitude/longitude WGS-84 coordinates (in degrees) to pixel XY coordinates at a
-- specified level of detail.
-- Adapted from https://learn.microsoft.com/en-us/bingmaps/articles/bing-maps-tile-system

CREATE OR REPLACE FUNCTION latlon_to_pixel_xy(
    IN latitude DOUBLE PRECISION,
    IN longitude DOUBLE PRECISION,
    IN level_of_detail INT,
    OUT pixel_x INT,
    OUT pixel_y INT
)
LANGUAGE plpgsql IMMUTABLE PARALLEL SAFE
AS $$
DECLARE
    _x DOUBLE PRECISION;
    _y DOUBLE PRECISION;
    _sin_latitude DOUBLE PRECISION;
    _map_size INT;
BEGIN
    latitude := clip(latitude, -85.05112878, 85.05112878);
    longitude := clip(longitude, -180, 180);

    _x := (longitude + 180) / 360;
    _sin_latitude := SIN(latitude * PI() / 180);
    _y := 0.5 - LN((1 + _sin_latitude) / (1 - _sin_latitude)) / (4 * PI());
    _map_size := map_size(level_of_detail);
    pixel_x := FLOOR(clip(_x * _map_size + 0.5, 0, _map_size - 1));
    pixel_y := FLOOR(clip(_y * _map_size + 0.5, 0, _map_size - 1));
END;
$$;
