-- Copyright (C) 2023:
--   Helmholtz-Zentrum Potsdam Deutsches GeoForschungsZentrum GFZ
--
-- This program is free software: you can redistribute it and/or modify it
-- under the terms of the GNU Affero General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or (at
-- your option) any later version.
--
-- This program is distributed in the hope that it will be useful, but
-- WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Affero
-- General Public License for more details.
--
-- You should have received a copy of the GNU Affero General Public License
-- along with this program. If not, see http://www.gnu.org/licenses/.

-- Converts a pixel from pixel XY coordinates at a specified level of detail into
-- latitude/longitude WGS-84 coordinates (in degrees).
-- Adapted from https://learn.microsoft.com/en-us/bingmaps/articles/bing-maps-tile-system

CREATE OR REPLACE FUNCTION pixel_xy_to_latlon(
    IN pixel_x INT,
    IN pixel_y INT,
    IN level_of_detail INT,
    OUT latitude DOUBLE PRECISION,
    OUT longitude DOUBLE PRECISION
)
LANGUAGE plpgsql IMMUTABLE PARALLEL SAFE
AS $$
DECLARE
    _x DOUBLE PRECISION;
    _y DOUBLE PRECISION;
    _map_size INT;
BEGIN
    _map_size := map_size(level_of_detail);
    _x := (clip(pixel_x, 0, _map_size - 1) / _map_size) - 0.5;
    _y := 0.5 - (clip(pixel_y, 0, _map_size - 1) / _map_size);
    latitude := 90.0 - 360.0 * ATAN(EXP(-_y * 2.0 * PI())) / PI();
    longitude := 360 * _x;
END;
$$;
