-- Copyright (C) 2023:
--   Helmholtz-Zentrum Potsdam Deutsches GeoForschungsZentrum GFZ
--
-- This program is free software: you can redistribute it and/or modify it
-- under the terms of the GNU Affero General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or (at
-- your option) any later version.
--
-- This program is distributed in the hope that it will be useful, but
-- WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Affero
-- General Public License for more details.
--
-- You should have received a copy of the GNU Affero General Public License
-- along with this program. If not, see http://www.gnu.org/licenses/.

-- Converts pixel XY coordinates into tile XY coordinates of the tile that contains the pixel.
-- Each tile is given XY coordinates ranging from (0, 0) in the north-west to ((2^zoom-level)–1,
-- (2^zoom-level)–1) in the south-east.
-- Adapted from https://learn.microsoft.com/en-us/bingmaps/articles/bing-maps-tile-system

CREATE OR REPLACE FUNCTION pixel_xy_to_tile_xy(
    IN pixel_x INT,
    IN pixel_y INT,
    OUT tile_x INT,
    OUT tile_y INT
)
LANGUAGE plpgsql IMMUTABLE PARALLEL SAFE
AS $$
BEGIN
    tile_x := pixel_x / 256;
    tile_y := pixel_y / 256;
END;
$$;
