-- Copyright (C) 2023:
--   Helmholtz-Zentrum Potsdam Deutsches GeoForschungsZentrum GFZ
--
-- This program is free software: you can redistribute it and/or modify it
-- under the terms of the GNU Affero General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or (at
-- your option) any later version.
--
-- This program is distributed in the hope that it will be useful, but
-- WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Affero
-- General Public License for more details.
--
-- You should have received a copy of the GNU Affero General Public License
-- along with this program. If not, see http://www.gnu.org/licenses/.

-- Converts a QuadKey into tile XY coordinates. Each tile is given XY coordinates ranging
-- from (0, 0) in the north-west to ((2^zoom-level)–1, (2^zoom-level)–1) in the
-- south-east.
-- Adapted from https://learn.microsoft.com/en-us/bingmaps/articles/bing-maps-tile-system

CREATE OR REPLACE FUNCTION quadkey_to_tile_xy(
    IN quadkey TEXT,
    OUT tile_x INT,
    OUT tile_y INT
)
LANGUAGE plpgsql IMMUTABLE PARALLEL SAFE
AS $$
DECLARE
    _level_of_detail INT;
    _mask INT;
BEGIN
    _level_of_detail := LENGTH(quadkey);
    tile_x := 0;
    tile_y := 0;
    FOR i IN REVERSE _level_of_detail..1 LOOP
        _mask = 1 << (i - 1);
        CASE SUBSTR(quadkey,_level_of_detail - i  + 1, 1)
            WHEN '0' THEN null;
            WHEN '1' THEN tile_x := tile_x | _mask;
            WHEN '2' THEN tile_y := tile_y | _mask;
            WHEN '3' THEN
                tile_x := tile_x | _mask;
                tile_y := tile_y | _mask;
            ELSE
                RAISE NOTICE 'Invalid QuadKey digit sequence.';
                RETURN;
        END CASE;
    END LOOP;
END;
$$;
